/**
 * @file ruter.cpp
 * @author Andreas Blakli
 */

#include "ruter.h"

using namespace std;

extern Stoppesteder* gStoppesteder;

/**
 * Constructor for Ruter, lager ett objekt.
 *
 */
Ruter::Ruter() {
    lesFraFil();
}

/**
 * Destructor for Ruter objekt.
 *
 */
Ruter::~Ruter() {
    skrivTilFil();
}

/**
 * Legger til en ny rute i map'et alleRuter basert på om
 * det er en buss elle bane rute.
 *
 * @param nr - Rutens unike nummer/ID.
 *
 * @see Buss::lesData().
 * @see Bane::lesData().
 */
void Ruter::ny(const int& nr) {
    cout << "\nBuss eller bane?" << '\n'
         << "\n1 - Buss" << '\n'
         << "2 - Bane" << '\n'
         << "0 - Avbryt" << '\n';

    switch (lesInt("", 0, 2)) {
        case 0:
            break;
        case 1: {
            Buss* buss = new Buss;
            buss->lesData();
            if (buss->ny(-1, -1) != nullptr)
                alleRuter.insert(pair<int, Rute*>(nr, buss));
            break;
        }
        case 2: {
            Bane* bane = new Bane;
            bane->lesData();
            if (bane->ny(-1, -1) != nullptr)
                alleRuter.insert(pair<int, Rute*>(nr, bane));
            break;
        }
        default:
            cout << "\nUlovlig valg!" << '\n';
            break;
    }
}

/**
 * Returnerer peker til ruten hvis den finnes.
 *
 * @param nr - Rutens nummer/ID.
 * @return Rute* - Peker til rute, nullptr hvis den ikke finnes.
 */
Rute* Ruter::hentEksisterende(const int nr) {
    Rute* rute = nullptr;
    for_each(begin(alleRuter), end(alleRuter),
             [&rute, &nr](pair<int const, Rute*>& mRuter) {
                 if (mRuter.first == nr) rute = mRuter.second;
             });
    return rute;
}

/**
 * Skriver meny valg til terminal.
 *
 */
void Ruter::skrivMeny() {
    cout << "\nN - Ny rute" << '\n'
         << "A - Skriv kort alle ruter" << '\n'
         << "E - Endre rute" << '\n'
         << "B - Rute beskrivelse" << '\n'
         << "T - Rutetabell" << '\n'
         << "Q - Tilbake" << '\n';
}

/**
 * Meny som velger hvilke funksjoner som skal brukes
 * basert på bruker valget gitt i valg parameteren.
 *
 * @param valg - Bruker valget.
 */
void Ruter::meny(const char& valg) {
    Rute* rute;
    int ruteNr;
    rute = nullptr;
    ruteNr = -1;

    switch (valg) {
        case 'N': {
            ruteNr = lesRute();
            rute = hentEksisterende(ruteNr);
            if (rute == nullptr) {
                ny(ruteNr);
            } else
                cout << "\nError: Rute eksisterer allerede!" << '\n';

            break;
        }
        case 'A':
            skrivKortData();
            break;
        case 'E':
            endre();
            break;
        case 'B':
            ruteNr = lesRute();
            spesifikkRuteData(ruteNr);
            break;
        case 'T':
            ruteNr = lesRute();
            rute = hentEksisterende(ruteNr);
            if (rute != nullptr) {
                rute->tabell();
            } else
                cout << "\nError: Rute eksisterer ikke!" << '\n';
            break;
        case 'Q':
            break;
        default:
            cout << "\nUgyldig valg!" << '\n';
            break;
    }
}

/**
 * Skriver ut info om en rute til terminal.
 *
 * @see Bane::skrivType().
 * @see Buss::skrivType().
 * @see Stoppesteder::hentEksisterende().
 * @see Stoppested::hentNavn().
 */
void Ruter::skrivKortData() {
    Buss* buss;
    Bane* bane;
    Stoppested* stopp;
    cout << "\nRute nummer\tType\tEndestasjon";
    for (const auto& val : alleRuter) {
        stopp = nullptr;
        cout << "\nRute nr:" << val.first;
        cout << '\t';
        val.second->skrivType();
        if (val.second->alleStoppestedenePaaRuten.size() > 0) {
            stopp = gStoppesteder->hentEksisterende(
                val.second->alleStoppestedenePaaRuten.back()->indexStoppested);
        }
        if (stopp != nullptr) cout << "\t" << stopp->hentNavn();
    }
}

/**
 * Skriver data forlengs og baklengs om en rute.
 *
 * @param rute - Peker til ruten som skal skrives ut.
 *
 * @see Rute::skrivData().
 */
void Ruter::skrivData(Rute* rute) {
    rute->skrivData();
}

/**
 * Skriver data forlengs og baklengs om en rute basert på rute
 * nummeret/ID.
 *
 * @param ruteNr - rute nummeret/ID.
 *
 * @see Rute::skrivData().
 */
void Ruter::spesifikkRuteData(const int& ruteNr) {
    Rute* rute;
    rute = hentEksisterende(ruteNr);
    if (rute != nullptr) {
        rute->skrivData();
    } else
        cout << "\nError: rute nr: " << ruteNr << " finnes ikke!" << '\n';
}

/**
 * Endrer en eksisternde rute.
 *
 * @see lesRute().
 * @see hentEksisterende().
 * @see spesifikkRuteData().
 * @see Rute::endre().
 */
void Ruter::endre() {
    Rute* rute;
    int ruteNr, valg;
    ruteNr = lesRute();
    rute = hentEksisterende(ruteNr);
    if (rute != nullptr) {
        spesifikkRuteData(ruteNr);
        rute->endre();
    } else
        cout << "\nError: rute nr: " << ruteNr << " finnes ikke!" << '\n';
}

/**
 * Leser inn rute nummer fra CLI.
 *
 * @return int - Innlest tall valgt av bruker.
 *
 * @see lesData3 lesInt().
 */
int Ruter::lesRute() {
    int ruteNr;
    ruteNr = lesInt("Skriv inn rute nummer", MIN_RUTER, MAX_RUTER);
    return ruteNr;
}

/**
 * Leser ruter fra fil.
 *
 * @see HjelpeFunksjoner::splittStreng().
 * @see Buss::lesFraFil().
 * @see Bane::lesFraFil().
 */
void Ruter::lesFraFil() {
    int antallRuter, i;
    antallRuter = i = 0;
    string type = "";
    vector<string> typeSplittet;
    ifstream fil(STI_RUTER);
    if (fil) {
        fil >> antallRuter;
        fil.ignore();
        while (!fil.eof() && i < antallRuter) {
            getline(fil, type);
            typeSplittet = hjelp.splittStreng('*', type);
            if (typeSplittet[1] == "Buss") {
                Buss* buss = new Buss;
                buss->lesFraFil(fil);
                alleRuter.insert(pair<int, Rute*>(stoi(typeSplittet.front()), buss));
            } else if (typeSplittet[1] == "Bane") {
                Bane* bane = new Bane;
                bane->lesFraFil(fil);
                alleRuter.insert(pair<int, Rute*>(stoi(typeSplittet.front()), bane));
            }
            i++;
        }
        fil.close();
    } else
        cout << "\nError: " << STI_RUTER << " ble ikke funnet!"
             << '\n';
}

/**
 * Skriver ruter til fil.
 *
 * @see Buss::skrivTilFil().
 * @see Bane::skrivTilFil().
 */
void Ruter::skrivTilFil() {
    ofstream fil(STI_RUTER);
    fil << alleRuter.size() << '\n';
    for (const auto& val : alleRuter) {
        fil << val.first << '*';
        val.second->skrivTilFil(fil);
        fil << "X" << '\n';
    }
}
